unit TP650;

interface

uses
SysUtils,
StrUtils,
Math;

type
  TCmd = packed record
    data : array[0..2047] of ansichar;
    len : Integer;
  end;
  TArrayAnsiString = array of AnsiString;


  TTP650 = Class(TObject)
  public
  function textoCondensado(texto: String): String;
  function textoNegrito(texto: String): AnsiString;
  function textoItalico(texto: String): AnsiString;
  function textoSublinhado(texto: String): AnsiString;
  function modoSublinhado(n : integer):AnsiString;
  function espacamentoPadrao(): AnsiString;
  function espacamentoCustom(n : integer): AnsiString;

  function qrcode(tamanho: Shortint; texto: AnsiString): AnsiString;
  function codEAN13(cod: AnsiString): AnsiString;
  function codUPCA(cod: AnsiString): AnsiString;
  function codEAN8(cod: AnsiString): AnsiString;
  function cod128B(cod: AnsiString): AnsiString;
  function setHeightCode(height: Integer): TBytes;
  function cortePapel(): TBytes;
  function abreGaveta(pino: Integer; pulso: Integer): AnsiString;
  function getStatusDrawer: AnsiString;
  function getStatusBotaoFeed: AnsiString;
  function getStatusTampa: AnsiString;
  function getStatusPapel: AnsiString;
  function statusDrawerOn(status: Integer): boolean;
  function statusBotaoFeedPressed(status: Integer): boolean;
  function statusTampa(status: Integer): boolean;
  function statusPapel(status: Integer): boolean;

  function logoNV(indice: Integer): AnsiString;
  function logoNVPrintPageMode(kc1 : Integer; kc2 : Integer): AnsiString;
  function logoNVDefinePageMode(kc1 : integer; kc2 : integer; hexa : AnsiString; width: integer; height : integer): AnsiString;

  function logoNVDeleteAllPageMode(): AnsiString;
  function logoNVDeleteSpecifiedPageMode(kc1 : integer; kc2 : integer): AnsiString;

  //function logoNVModePage(kc1: Integer; kc2: Integer; x:integer; y: integer): AnsiString;



  function setWidthCode(width: Integer): TBytes;
  function cod128C(cod: AnsiString): TBytes;


  function ligaModoPagina():TBytes;
  function desligaModoPagina():TBytes;
  function ajustarDirecao():TBytes;
  function printMemory():TBytes;
  function ajustarArea(xL : integer; xH : integer; yL : integer; yH : integer; dxL : integer; dxH : integer; dyL : integer; dyH : integer):TBytes;
  function printMode(texto: AnsiString ;bit : AnsiString): AnsiString;
  function justify(j : integer):TBytes;
  function imgPageMode():AnsiString;


  //function gravarLogoRasterStr(const RasterStr: AnsiString; flag: integer ; KeyCode1: Byte; AWidth: Integer; AHeight: Integer): AnsiString;
  end;

implementation

function TTP650.cortePapel(): TBytes;
begin
	//Result := #27+#109;

  SetLength(result, 2);
  result[0] := 27;
  result[1] := 109;

end;

function TTP650.espacamentoPadrao(): AnsiString;
begin
	Result := #27+#50;
end;

function TTP650.espacamentoCustom(n : integer): AnsiString;
begin
	Result := #27+#51+chr(n);
end;
function TTP650.modoSublinhado(n : integer): AnsiString;
begin
	Result := #27+#45+chr(n);
end;
function TTP650.abreGaveta(pino: Integer; pulso: Integer): AnsiString;
begin
	Result := #16+#20+#01 + chr(pino) + chr(pulso)
end;

function TTP650.codEAN13(cod: AnsiString): AnsiString;
begin
	Result := #29+#107+#02 + cod + #00;
end;

function TTP650.codUPCA(cod: AnsiString): AnsiString;
begin
	Result := #29+#107+#00 + cod + #00;
  end;

function TTP650.codEAN8(cod: AnsiString): AnsiString;
begin
	Result := #29+#107+#03 + cod + #00;
end;

function TTP650.cod128B(cod: AnsiString): AnsiString;
var
  len : integer;
  i : integer;
begin
  if(length(cod) >30) then
  begin
    i := 0;
    len := Round(length(cod)/2);
    Result := #29+#107+#73 + chr(len+2) + #123+#66 + copy(cod,0,len) + #29+#107+#73 + chr(len+2) + #123+#66 + copy(cod,len+1,len*2+1);
  end
  else
  begin
     Result := #29+#107+#73 + chr(length(cod)+2) + #123+#66 + cod;
  end;

	//Result := #29+#107+#73 + chr(length(cod)+2) + #123+#65 + cod;
end;

procedure initArray(b : array of Char; c : char; size : integer);
var
  i: integer;
begin
  i :=1;
    while( i <= size)do
    begin
      b[i] := c;
      i := i + 1 ;
    end;
end;

function StrToHex(S: String): string;
var I: Integer;
begin
  Result:= '';
  for I := 1 to length (S) do
    Result:= Result+IntToHex(ord(S[i]),2);
end;

function HexToStr(H: String): String;
var I: Integer;
begin
  Result:= '';
  for I := 1 to length (H) div 2 do
    Result:= Result+Char(StrToInt('$'+Copy(H,(I-1)*2+1,2)));
end;

function convertCODE128C(b : TArrayAnsiString; cod : AnsiString): TArrayAnsiString;
var
  i: integer;
  cont : integer;
  tlB : integer;
  str : AnsiString;
begin

    i :=1;cont := 0;str := ''; tlB:=0;
    while( i <= Length(cod))do
    begin
       if(cont = 2) then
       begin
          b[tlB] := StrToHex(str);
          str := '';
          cont := 0; tlB := tlB + 1;
       end
       else
       begin
         str := str + cod[i];
         i := i + 1; cont := cont + 1;
       end
    end;
    //str :=  cod[3] + cod[4];
    //b[0] :=  str;
    Result := b;
end;

function TTP650.cod128C(cod: AnsiString): TBytes;
var
  i, len, n, x: integer;
  s : AnsiString;
begin
  len := Length(cod);
  if len = 0 then exit;
  s := cod;
  if Odd(len) then
  begin
    s := '0'+s;
    inc(len);
  end;
  len := len div 2;
  SetLength(result, 6 + len);

  result[0] := 29;
  result[1] := 107;
  result[2] := 73;
  result[3] := 2 + len;
  result[4] := 123;
  result[5] := 67;

  n := length(s);

  i := 1;
  x := 6;

  while i < n do
  begin
    result[x] := StrToInt(MidStr(s, i, 2));
    inc(i, 2);
    inc(x, 1);
  end;

end;

function TTP650.setWidthCode(width: Integer): TBytes;
begin
  SetLength(result, 3);
  result[0] := 29;
  result[1] := 119;
  result[2] := width mod 256;
end;

{function TTP650.ajustarArea(mode : AnsiString):AnsiString;
begin
    Result := #27+#87+#49;
end;     }

function TTP650.ajustarDirecao():TBytes;
begin

 SetLength(result, 3);
  result[0] := 27;
  result[1] := 84;
  result[2] := 48;
end;

function TTP650.ajustarArea(xL : integer; xH : integer; yL : integer; yH : integer; dxL : integer; dxH : integer; dyL : integer; dyH : integer):TBytes;
begin

 SetLength(result, 10);
  result[0] := 27;
  result[1] := 87;
  result[2] := xL;  //xL
  result[3] := xH;  //xH
  result[4] := yL;  //yL
  result[5] := yH;  //yH
  result[6] := dxL; //dxL
  result[7] := dxH; //dxH
  result[8] := dyL; //dyL
  result[9] := dyH; //dyH



  //x0 = [(xL + xH256)  0.125 mm]       Posio inicial Horizontal
  //y0 = [(yL + yH  256)  0.125 mm]     Posio inicial Vertical
  //dx = [(dxL + dxH 256) 0.125 mm]     Largura da area de impressao
  //dy = [(dyL + dyH  256) 0.125 mm]    Altura da area de impressao


end;

function TTP650.printMemory():TBytes;
begin

  SetLength(result, 2);
  result[0] := 27;
  result[1] := 12;

end;

function TTP650.ligaModoPagina():TBytes;
begin
  SetLength(result, 2);
  result[0] := 27;
  result[1] := 76;
end;

function TTP650.desligaModoPagina():TBytes;
begin
  SetLength(result, 2);
  result[0] := 27;
  result[1] := 83;
end;

function TTP650.justify(j : integer):TBytes;
begin
  SetLength(result, 3);

  result[0] := 27;
  result[1] := 97;
  result[2] := j;
end;

function TTP650.setHeightCode(height : Integer): TBytes;
begin
	//Result := #29+#104+chr(altura mod 256);

  SetLength(result, 3);

  result[0] := 29;
  result[1] := 104;
  result[2] := height mod 256;
end;

function TTP650.qrcode(tamanho: Shortint; texto: AnsiString): AnsiString;
var
  l: Integer;
  f65: AnsiString;
  f67: AnsiString;
  f69: AnsiString;
  f80: AnsiString;
  f81: AnsiString;
begin
  //f 65 mode
	f65 := #29+#40+#107+#04+#00+#49+#65+#50+#00;
	//f67 size
	f67 := #29+#40+#107+#03+#00+#49+#67 + Chr(tamanho mod 17);
	//f 69 - ec level
	f69 := #29+#40+#107+#03+#00+#49+#69+#03;
	//f 80 - save data
	l := length(texto) + 3;
	f80 := #29+#40+#107 + AnsiChar(l mod 256) + AnsiChar(l div 256) + #49+#80+#48;
	//f 81 - print
	f81 := #29+#40+#107+#03+#00+#49+#81+#48;
  //Move(f65[1],q.data,
  Result := f65 + f67 + f69 + f80 + texto + f81;

end;

function TTP650.imgPageMode():AnsiString;
begin
  Result := #29+#40+#76+#6+#0+#48+#69+#32+#32+#1+#1+#13+#10+#29+#56+#42+#01;
  //1D 28 4C 06 00 30 45 20 20 01 01 0D 0A 1D 56 42 01
end;

function TTP650.textoCondensado(texto: String): String;
begin
  Result := #27+#33+#00+#27+#77+#01 + texto + #27+#77+#00;
end;

function TTP650.textoNegrito(texto: String): AnsiString;
begin
  Result := #27+#33+#00+#27+#71+#255 + texto + #27+#71+#00;
end;

function TTP650.printMode(texto: AnsiString; bit : AnsiString): AnsiString;
begin
  Result := #27+#33+bit+texto;
end;



function TTP650.textoItalico(texto: String): AnsiString;
begin
  Result := #27+#33+#00+#27+#53+#255 + texto + #27+#53+#00;
end;
function TTP650.textoSublinhado(texto: String): AnsiString;
begin
  Result := #27+#33+#00+#27+#49+#01+ texto + #27+#49+#00;
end;






function TTP650.getStatusDrawer: AnsiString;
begin
  Result := #16+#04+#1;
end;

function TTP650.getStatusBotaoFeed: AnsiString;
begin
	Result := #16+#04+#2;
end;

function TTP650.getStatusTampa: AnsiString;
begin
	Result := #16+#04+#2;
end;


function TTP650.getStatusPapel: AnsiString;
begin
	Result := #16+#04+#04;
end;

function AjustarKeyCodeUnico(AKeyCode: Byte): Byte;
begin
  if (AKeyCode >= 48) and (AKeyCode <= 57) then  // '0'..'9'
    Result := StrToInt( chr(AKeyCode) )
  else
    Result := AKeyCode;
end;

{function comandoGravarLogoColumnStr(const RasterStr: AnsiString; AWidth: Integer; AHeight: Integer; KeyCode: Byte): AnsiString;
var
  SLColumnStr : TStrings;
  HeightInBytes, LenBuffer, LenSLineBreak, LenK: Integer;
  Buffer: AnsiString;
begin
  {if KeyCode <> 1 then
    raise Exception.CreateFmt('Gravando Logo em modo Legado (FS q), somente aceita a posicao "1"');

  SLColumnStr := TStringList.Create;
  try
    RasterStrToColumnStr(RasterStr, AWidth, SLColumnStr, 0);
    Buffer := SLColumnStr.Text;
    LenSLineBreak := Length(sLineBreak);
    LenBuffer := Length(Buffer);
    Delete(Buffer, LenBuffer-LenSLineBreak+1, LenSLineBreak) ;  // Remove sLineBreak do final
  finally
    SLColumnStr.Free;
  end;

  HeightInBytes := ceil(AHeight/8);
  LenBuffer := Length(Buffer);
  LenK := AWidth*(HeightInBytes*8);
  Buffer := Buffer + StringOfChar(#0, max(LenK - LenBuffer,0));

  Result := #28 + #113 +      // Comando (obsoleto) para grava??o de NV ABit Image (em modo coluna)
                  #1 +      // Uma Imagem
                  AnsiChar(AWidth)  +
                  AnsiChar(HeightInBytes) +
                  Buffer;
end;

procedure TACBrEscPosEpson.VerificarKeyCodes;
  procedure VerificarKeyCode(AkeyCode: Integer; NomeKeyCode: String);
  begin
    if (AKeyCode < 32) or (AkeyCode > 126) then
      raise EPosPrinterException.Create(NomeKeyCode+' deve estar entre 32 a 126');
  end;
begin
  with fpPosPrinter.ConfigLogo do
  begin
    VerificarKeyCode(KeyCode1, 'KeyCode1');
    VerificarKeyCode(KeyCode2, 'KeyCode2');
  end;
end;  }

{function TTP650.gravarLogoRasterStr(flag: integer ; const RasterStr: AnsiString;  AWidth: Integer; AHeight: Integer; KeyCode1: Byte): AnsiString;
var
  KeyCode: Byte;
begin
   if (flag = 0) then //Comando Legado FS
    begin
      KeyCode := AjustarKeyCodeUnico(KeyCode1);
      Result := comandoGravarLogoColumnStr(RasterStr, AWidth, AHeight, KeyCode);
    end
    else //Comando novo GS
    begin
      VerificarKeyCodes;

      // Comando para gravar Raster Img na mem?ria
      Result := #48 + #67 + #48 +   // m + fn + a
                AnsiChr(KeyCode1) + AnsiChr(KeyCode2) +
                #1 +                // b - 1 Cor, Mono
                IntToLEStr(AWidth)  +
                IntToLEStr(AHeight) +
                '1' +               // Inicio cor 1
                RasterStr +
                #1;                 // Fim, cor 1

      Result := GS + '8L' + IntToLEStr(Length(Result), 4) + Result;
    end;
end;  }

function TTP650.logoNV(indice: Integer): AnsiString;
begin
  Result := #28+#112+AnsiChar(indice)+#00;
end;

function HexToInt(Hexadecimal : String) : Integer;
begin
  Result := StrToInt('$' + Hexadecimal);
end;
function TTP650.logoNVPrintPageMode(kc1 : Integer; kc2 : Integer): AnsiString;
var
  ind1 : integer;
  ind2 : integer;
  teste : String;
begin
  //kc1 :=

  //ind1 := HexToInt(kc1);
  //ind2 := HexToInt(kc2);
  //teste :=  intToHex(ind1,8);
  Result := #29+#40+#76+#06+#00+#48+#69+AnsiChar(kc1)+AnsiChar(kc2)+#1+#1;



  {SetLength(result, 11);

  result[0] := 29;
  result[1] := 40;
  result[2] := 76;
  result[3] := 06;
  result[4] := 00;
  result[5] := 48;
  result[6] := 69;
  result[7] := intToHex(ind1,8);
  result[8] := intToHex(ind2,8);
  result[9] := 1;
  result[10] :=1;
  //1D  28 4C   06  00  30  45  20  20 01 01;}



end;
function TTP650.logoNVDeleteAllPageMode(): AnsiString;
begin
  Result := #29+#40+#76+#5+#0+#48+#65+#67+#76+#82;
           //1D  28 4C  05 00  30  41 43  4C 52
end;

function TTP650.logoNVDeleteSpecifiedPageMode(kc1 : integer; kc2 : integer): AnsiString;
begin
  Result := #29+#40+#76+#4+#0+#48+#66+AnsiChar(kc1)+AnsiChar(kc2);
           //1D  28 4C  04 00  30  42  kc1 kc2
end;

function TTP650.logoNVDefinePageMode(kc1 : integer; kc2 : integer; hexa : AnsiString; width: integer; height : integer): AnsiString;
var

  pL, pH : AnsiString;
  teste : integer;
begin
  pL := #147; //96
  pH := #2;   //2
  //pL := AnsiChar(147);

  Result := #29+#40+#76+pL+pH+#48+#67+#48+ AnsiChar(kc1)  +AnsiChar(kc2)    +#1+AnsiChar(width)+#0+ AnsiChar(height)+#0+#49 + hexa;
           //1D  28  4C        30  43  30  kc1   kc2    01  ?  00   ?  00  31
           //GS  (   L  pl ph  m   fn  a   kc1   kc2    b  xl  xh  yl  yh

           //talvez para mexer no alinhamento, seja vertical ou horizontal, alterar xh, yh...


  {SetLength(result, 17);

  result[0] := 29;
  result[1] := 40;
  result[2] := 76;
  result[3] := 147;
  result[4] := 2;
  result[5] := 48;
  result[6] := 67;
  result[7] := 48;
  result[8] := 32;
  result[9] := 32;
  result[10] :=1;
  result[11] :=72;
  result[12] :=0;
  result[13] :=68;
  result[14] :=0;
  result[15] :=48;
  result[16] := hexa;}


end;

{function TTP650.logoNVModePage(kc1: Integer; kc2: Integer; x:integer; y: integer): AnsiString;
begin
  Result := #29+#40+#76+#6+#0+#48+#69+AnsiChar(kc1)+AnsiChar(kc2)+AnsiChar(x)+AnsiChar(y);
end;  }

function TTP650.statusDrawerOn(status: Integer): boolean;
begin
	//Result := (status and 23) <> 23;
  Result := status = 22;
end;

function TTP650.statusBotaoFeedPressed(status: Integer): boolean;
begin
	Result := (status and 31) = 26;
end;

function TTP650.statusTampa(status: Integer): boolean;
begin
	Result := status <> 54;
end;

function TTP650.statusPapel(status: Integer): boolean;
begin
	Result := (status and 115) <> 114;
end;



end.
